package com.tecnavia.dailymotion.sdk

import android.app.Dialog
import android.content.Context
import android.content.pm.ActivityInfo
import android.util.Log
import android.view.ViewGroup
import android.widget.FrameLayout
import com.dailymotion.player.android.sdk.Dailymotion
import com.dailymotion.player.android.sdk.PlayerParameters
import com.dailymotion.player.android.sdk.PlayerView
import com.dailymotion.player.android.sdk.webview.error.PlayerError
import com.dailymotion.player.android.sdk.webview.events.PlayerEvent
import com.facebook.react.bridge.WritableMap
import com.facebook.react.uimanager.ThemedReactContext
import com.facebook.react.uimanager.events.RCTEventEmitter
import com.tecnavia.dailymotion.sdk.listeners.EventHandler
import com.tecnavia.dailymotion.sdk.listeners.PlayerEventListener
import java.lang.Exception
import java.util.HashMap

class TaPlayer(context: Context?) : FrameLayout(context!!), EventHandler {
    private var fullDialog: Dialog? = null
    private var isPlayerFullscreen: Boolean = false
    private lateinit var playerData: HashMap<String, Any>
    private var player: PlayerView? = null
    private var mEventEmitter: RCTEventEmitter? = null

    constructor(context: Context?, mEventEmitter: RCTEventEmitter) : this(context) {
        this.mEventEmitter = mEventEmitter
    }

    fun createInstance() {
        if (player == null) {

            val playerId = playerData["playerId"] as String
            val videoId = playerData["video"] as String

            val playerParameters =
                    PlayerParameters(
                            mute = playerData["mute"] as? Boolean ?: false,
                            startTime = playerData["startTime"] as? Long ?: 0,
                    )
            val playerEventListener = PlayerEventListener(mEventEmitter, this, this);

            Dailymotion.createPlayer(
                context = context,
                playerId = playerId,
                videoId = videoId,
                playerParameters = playerParameters,
                playerSetupListener =
                object : Dailymotion.PlayerSetupListener {
                    override fun onPlayerSetupSuccess(dmPlayer: PlayerView) {
                        Log.d("TaPlayer", "Successfully created dailymotion player")

                        val lp = LayoutParams(
                            LayoutParams.MATCH_PARENT,
                            LayoutParams.MATCH_PARENT
                        )

                        addView(dmPlayer, lp)

                        player = dmPlayer

                        Log.d("TaPlayer", "Added dailymotion player to view hierarchy")

                        playerEventListener.onPlayerLoaded(player!!)
                    }

                    override fun onPlayerSetupFailed(error: PlayerError) {
                        Log.e(
                            "TaPlayer",
                            "Error while creating dailymotion player: ${error.message}"
                        )
                    }
                },
                playerListener = playerEventListener,
                videoListener = playerEventListener
            )
        }
    }

    fun setPlayerData(map: HashMap<String, Any>) {
        playerData = map
    }

    fun load() {
        createInstance()

        if (player != null) {
            val videoId = playerData["video"] as String
            val playlistId = playerData["playlistId"] as? String
            val startTime = playerData["startTime"] as? Long ?: 0

            player!!.loadContent(videoId, playlistId, startTime)
        }

    }

    fun getPlayer(): PlayerView? {
        return player
    }

    fun pause() {
        player?.pause()
    }

    fun play() {
        player?.play()
    }

    fun seek(time: Int) {
        player?.seekTo(time.toLong())
    }

    fun mute() {
        player?.setMute(true)
    }

    fun unmute() {
        player?.setMute(false)
    }

    fun release() {
        player?.destroy()
    }

    fun toggleFullscreen() {
        try {
            isPlayerFullscreen = !isPlayerFullscreen
            toggleOrientation()
            if (isPlayerFullscreen) {
                enableFullscreen()
            } else {
                disableFullscreen()
            }
            player!!.setFullscreen(isPlayerFullscreen)
        } catch (e: Exception) {
            e.printStackTrace()
            isPlayerFullscreen = false
            toggleOrientation()
        }
    }

    private fun enableFullscreen() {
        removePlayerFromCurrentView()
        val activity = (context as ThemedReactContext).currentActivity
        activity?.runOnUiThread {
            if (!activity.isFinishing) {
                fullDialog = Dialog(activity, R.style.DailyMotionFullscreen)
                fullDialog?.setContentView(player!!)
                fullDialog?.setCancelable(false)
                if (!fullDialog?.isShowing!!) {
                    fullDialog?.show()
                    restartPlayer()
                }
            }
        }
    }

    private fun disableFullscreen() {
        removePlayerFromCurrentView()
        addView(player)
        fullDialog?.hide()
        restartPlayer()
    }

    private fun toggleOrientation() {
        try {
            if (isPlayerFullscreen)
                    (context as ThemedReactContext).currentActivity?.requestedOrientation =
                            ActivityInfo.SCREEN_ORIENTATION_SENSOR_LANDSCAPE
            else
                    (context as ThemedReactContext).currentActivity?.requestedOrientation =
                            ActivityInfo.SCREEN_ORIENTATION_UNSPECIFIED
        } catch (npe: NullPointerException) {
            npe.printStackTrace()
        }
    }

    private fun removePlayerFromCurrentView() {
        (player?.parent as ViewGroup).removeView(player)
    }

    private fun restartPlayer() {
        player?.getState(object : PlayerView.PlayerStateCallback {
            override fun onPlayerStateReceived(
                playerView: PlayerView,
                playerState: PlayerEvent.PlayerState
            ) {
                if (!playerState.playerIsPlaying!!) {
                    play()
                }
            }
        })
    }

    override fun onConsume(map: WritableMap?): Boolean {
        if ("fullscreen_toggle_requested" == map?.getString("name")) {
            toggleFullscreen()
            return true
        }
        return false
    }
}
