function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
import React, { useEffect, useImperativeHandle, useRef, useState } from "react";
import PropTypes from "prop-types";
import { Animated, Dimensions, findNodeHandle, NativeModules, Platform, requireNativeComponent, UIManager, View } from "react-native";
const NATIVE_VIEW_NAME = "RCTDailyMotionPlayer";
const COMMANDS = {
  PLAY: "play",
  PAUSE: "pause",
  SEEK: "seek",
  MUTE: "mute",
  UNMUTE: "unmute",
  RELEASE: "release",
  FULLSCREEN_BUTTON: "fullscreenButton"
};
const RCTDailymotionPlayer = requireNativeComponent(NATIVE_VIEW_NAME);

/**
 * @param {RCTDailymotionPlayer} ref native view reference
 * @param {String} command command name
 * @param {Array} params ordered array params
 */
const execCommand = (ref, command, params = []) => {
  try {
    const viewId = findNodeHandle(ref.current);
    const commands = UIManager.getViewManagerConfig(NATIVE_VIEW_NAME).Commands;
    const commandId = commands[command];
    UIManager.dispatchViewManagerCommand(viewId, Platform.select({
      ios: commandId,
      android: commandId.toString()
    }), params);
  } catch (e) {
    console.warn(e.message);
  }
};

/**
 * render DailyMotionPlayer
 * expose methods to interact with the player
 * manage fullscreens state
 */
const DailyMotionPlayer = /*#__PURE__*/React.forwardRef((props, ref) => {
  const {
    events,
    style,
    playerStyle: playerStyleProp,
    ...rest
  } = props;

  //!patch as on first render in Android we are unable to get the correct size from parent
  const defaultStyle = Platform.select({
    ios: playerStyleProp,
    android: {}
  });
  const [playerStyle, setPlayerStyle] = useState(defaultStyle);
  useImperativeHandle(ref, () => ({
    play: () => execCommand(playerRef, COMMANDS.PLAY),
    pause: () => execCommand(playerRef, COMMANDS.PAUSE),
    seek: seconds => execCommand(playerRef, COMMANDS.SEEK, [seconds]),
    mute: () => execCommand(playerRef, COMMANDS.MUTE),
    unmute: () => execCommand(playerRef, COMMANDS.UNMUTE),
    release: () => execCommand(playerRef, COMMANDS.RELEASE),
    setFullscreenButton: () => execCommand(playerRef, COMMANDS.FULLSCREEN_BUTTON)
  }));
  const playerRef = useRef(null);
  const onError = ({
    nativeEvent
  }) => {
    const {
      onPlayerError
    } = events;
    if (typeof onPlayerError === "function") {
      onPlayerError(nativeEvent);
    }
  };

  /**
   * map event to specific callback or to generic if no one is passed
   * @see https://developers.dailymotion.com/player/#embed-mobile-android-events to map other events
   */
  const onEvent = ({
    nativeEvent
  }) => {
    const {
      onPlayerEvent,
      onPlayerLoaded,
      onStart,
      onWaiting,
      onPlay,
      onPlaying,
      onPause,
      onSeeking,
      onSeeked,
      onEnd
    } = events;
    if (nativeEvent.name === "loaded") {
      if (Platform.OS === "android") {
        setPlayerStyle(playerStyleProp);
      }
      if (typeof onPlayerLoaded === "function") {
        onPlayerLoaded(nativeEvent);
      }
    } else if (nativeEvent.name === "start" && typeof onStart === "function") {
      onStart(nativeEvent);
    } else if (nativeEvent.name === "waiting" && typeof onWaiting === "function") {
      onWaiting(nativeEvent);
    } else if (nativeEvent.name === "play" && typeof onPlay === "function") {
      onPlay(nativeEvent);
    } else if (nativeEvent.name === "playing" && typeof onPlaying === "function") {
      onPlaying(nativeEvent);
    } else if (nativeEvent.name === "pause" && typeof onPause === "function") {
      onPause(nativeEvent);
    } else if (nativeEvent.name === "seeking" && typeof onSeeking === "function") {
      onSeeking(nativeEvent);
    } else if (nativeEvent.name === "seeked" && typeof onSeeked === "function") {
      onSeeked(nativeEvent);
    } else if (nativeEvent.name === "end" && typeof onEnd === "function") {
      onEnd(nativeEvent);
    } else if (typeof onPlayerEvent === "function") {
      onPlayerEvent(nativeEvent);
    }
  };
  return /*#__PURE__*/React.createElement(View, {
    style: style
  }, /*#__PURE__*/React.createElement(RCTDailymotionPlayer, _extends({
    ref: playerRef
  }, rest, {
    style: playerStyle,
    onPlayerEvent: onEvent,
    onPlayerError: onError
  })));
});
DailyMotionPlayer.propTypes = {
  style: PropTypes.any,
  playerStyle: PropTypes.any,
  events: PropTypes.shape({
    onStart: PropTypes.func,
    onWaiting: PropTypes.func,
    onPlay: PropTypes.func,
    onPlaying: PropTypes.func,
    onSeeking: PropTypes.func,
    onSeeked: PropTypes.func,
    onEnd: PropTypes.func,
    onPlayerEvent: PropTypes.func,
    onPlayerError: PropTypes.func,
    onPlayerLoaded: PropTypes.func
  })
};
DailyMotionPlayer.defaultProps = {
  fullscreenSize: {},
  events: {},
  playerStyle: {
    flex: 1
  }
};
export default DailyMotionPlayer;
//# sourceMappingURL=index.js.map